@@ -47,6 +47,13 @@
 #include <string.h> /* for strdup */
 #include <limits.h> /* for INT_MAX */
 
+#ifdef WIN32
+#define strncasecmp      strnicmp
+#ifndef HAVE_SNPRINTF
+#include "vsnprintf.h"
+#endif
+#endif
+
 struct icalcomponent_impl 
 {
 	char id[5];
@@ -247,24 +254,22 @@ icalcomponent_free (icalcomponent* c)
 
     if(c != 0 ){
        
-		if ( c->properties != 0 )
-		{
-		   while( (prop=pvl_pop(c->properties)) != 0){
-		   assert(prop != 0);
-			   icalproperty_set_parent(prop,0);
-		   icalproperty_free(prop);
-		   }
-	        pvl_free(c->properties);
-		}
+	if ( c->properties != 0 )
+	{
+	    while( (prop=pvl_pop(c->properties)) != 0){
+		icalproperty_set_parent(prop,0);
+		icalproperty_free(prop);
+	    }
+	    pvl_free(c->properties);
+	}
       
 
-       while( (comp=pvl_data(pvl_head(c->components))) != 0){
-	   assert(comp!=0);
-	   icalcomponent_remove_component(c,comp);
-	   icalcomponent_free(comp);
-       }
+	while( (comp=pvl_data(pvl_head(c->components))) != 0){
+	    icalcomponent_remove_component(c,comp);
+	    icalcomponent_free(comp);
+	}
        
-       pvl_free(c->components);
+	pvl_free(c->components);
 
 	if (c->x_name != 0) {
 	    free(c->x_name);
@@ -433,8 +438,13 @@ icalcomponent_remove_property (icalcomponent* component, icalproperty* property)
     icalerror_check_arg_rv( (component!=0), "component");
     icalerror_check_arg_rv( (property!=0), "property");
     
+#ifdef ICAL_REMOVE_NONMEMBER_COMPONENT_IS_ERROR
     icalerror_assert( (icalproperty_get_parent(property)),"The property is not a member of a component");
-
+#else
+    if(icalproperty_get_parent(property) == 0){
+	return;
+    }
+#endif
     
     for( itr = pvl_head(component->properties);
 	 itr != 0;
@@ -866,7 +876,9 @@ int icalproperty_recurrence_is_excluded(icalcomponent *comp,
 	 
     struct icaltimetype exdatetime = icalcomponent_get_datetime(comp, exdate);
 
-    if (icaltime_compare(*recurtime, exdatetime) == 0) {
+    if ((icaltime_is_date(exdatetime) 
+    &&   icaltime_compare_date_only(*recurtime, exdatetime) == 0) ||
+        (icaltime_compare(*recurtime, exdatetime) == 0)) {
       /** MATCHED **/
         
       comp->property_iterator = property_iterator;
@@ -1057,6 +1069,10 @@ void icalcomponent_foreach_recurrence(icalcomponent* comp,
       if (icaltime_is_null_time(rrule_time)) 
 	break;
 
+      /* if we have iterated past end time, then no need to check any further */
+      if (icaltime_compare(rrule_time, end) > 0)
+	break;
+
       dur = icaltime_subtract(rrule_time, dtstart);
 
       recurspan.start = basespan.start + icaldurationtype_as_int(dur);
@@ -1102,6 +1118,7 @@ void icalcomponent_foreach_recurrence(icalcomponent* comp,
 
     if (!icalproperty_recurrence_is_excluded(comp, &dtstart, &rdate_period.time)) {
       /** call callback action **/
+      if (icaltime_span_overlaps(&recurspan, &limit_span))
       (*callback) (comp, &recurspan, callback_data);
     }
     comp->property_iterator = property_iterator;
@@ -1171,6 +1188,8 @@ void icalcomponent_strip_errors(icalcomponent* component)
 	if(icalproperty_isa(p) == ICAL_XLICERROR_PROPERTY)
 	{
 	    icalcomponent_remove_property(component,p);
+	    icalproperty_free(p);
+	    p = NULL;
 	}
     }
     
@@ -1238,6 +1257,8 @@ void icalcomponent_convert_errors(icalcomponent* component)
 					   icalproperty_new_requeststatus(rst));
 		
 		icalcomponent_remove_property(component,p);
+		icalproperty_free(p);
+		p = NULL;
 	    }
 	}
     }
@@ -2259,6 +2280,7 @@ icalcomponent_handle_conflicting_vtimezones (icalcomponent *comp,
           free(tzid_copy);
 	} else {
 	  icalarray_append (tzids_to_rename, tzid_copy);
+	  free(tzid_copy);
 	  icalarray_append (tzids_to_rename, existing_tzid_copy);
 	}
 	return;
@@ -2294,6 +2316,8 @@ icalcomponent_handle_conflicting_vtimezones (icalcomponent *comp,
   strcpy (new_tzid + tzid_len, suffix_buf);
   icalarray_append (tzids_to_rename, tzid_copy);
   icalarray_append (tzids_to_rename, new_tzid);
+  free(tzid_copy);
+  free(new_tzid);
 }
 
 
