@@ -40,6 +40,11 @@
 #include "icalmemory.h"
 #include "icalvalue.h"
 
+#ifdef WIN32
+#ifndef HAVE_SNPRINTF
+#include "vsnprintf.h"
+#endif
+#endif
 
 
 
@@ -79,7 +84,6 @@ struct icaldurationtype icaldurationtype_from_string(const char* str)
     int begin_flag = 0;
     int time_flag = 0;
     int date_flag = 0;
-    int week_flag = 0;
     int digits=-1;
     int scan_size = -1;
     int size = strlen(str);
@@ -137,19 +141,19 @@ struct icaldurationtype icaldurationtype_from_string(const char* str)
 		}
 
 	    case 'H': {	
-		if (time_flag == 0||week_flag == 1||d.hours !=0||digits ==-1) 
+		if (time_flag == 0||d.hours !=0||digits ==-1) 
 		    goto error;
 		d.hours = digits; digits = -1;
 		break;
 	    }
 	    case 'M': {
-		if (time_flag == 0||week_flag==1||d.minutes != 0||digits ==-1) 
+		if (time_flag == 0||d.minutes != 0||digits ==-1) 
 		    goto error;
 		d.minutes = digits; digits = -1;	    
 		break;
 	    }
 	    case 'S': {
-		if (time_flag == 0||week_flag==1||d.seconds!=0||digits ==-1) 
+		if (time_flag == 0||d.seconds!=0||digits ==-1) 
 		    goto error;
 		d.seconds = digits; digits = -1;	    
 		break;
@@ -157,12 +161,11 @@ struct icaldurationtype icaldurationtype_from_string(const char* str)
 	    case 'W': {
 		if (time_flag==1||date_flag==1||d.weeks!=0||digits ==-1) 
 		    goto error;
-		week_flag = 1;	
 		d.weeks = digits; digits = -1;	    
 		break;
 	    }
 	    case 'D': {
-		if (time_flag==1||week_flag==1||d.days!=0||digits ==-1) 
+		if (time_flag==1||d.days!=0||digits ==-1) 
 		    goto error;
 		date_flag = 1;
 		d.days = digits; digits = -1;	    
@@ -199,8 +202,17 @@ void append_duration_segment(char** buf, char** buf_ptr, size_t* buf_size,
 
 char* icaldurationtype_as_ical_string(struct icaldurationtype d) 
 {
+	char *buf;
+	buf = icaldurationtype_as_ical_string_r(d);
+	icalmemory_add_tmp_buffer(buf);
+	return buf;
+}
 
-    char *buf, *output_line;
+
+char* icaldurationtype_as_ical_string_r(struct icaldurationtype d) 
+{
+
+    char *buf;
     size_t buf_size = 256;
     char* buf_ptr = 0;
     int seconds;
@@ -248,11 +260,7 @@ char* icaldurationtype_as_ical_string(struct icaldurationtype d)
 	icalmemory_append_string(&buf, &buf_ptr, &buf_size, "PT0S");
     }
  
-    output_line = icalmemory_tmp_copy(buf);
-    icalmemory_free_buffer(buf);
-
-    return output_line;
-    
+    return buf;
 }
 
 
