@@ -174,14 +174,17 @@ icalproperty* icalproperty_new_from_string(const char* str)
 {
 
     size_t buf_size = 1024;
-    char* buf = icalmemory_new_buffer(buf_size);
-    char* buf_ptr = buf;  
+    char* buf;
+    char* buf_ptr;
     icalproperty *prop;
     icalcomponent *comp;
     int errors  = 0;
 
     icalerror_check_arg_rz( (str!=0),"str");
 
+    buf = icalmemory_new_buffer(buf_size);
+    buf_ptr = buf;
+
     /* Is this a HACK or a crafty reuse of code? */
 
     icalmemory_append_string(&buf, &buf_ptr, &buf_size, "BEGIN:VCALENDAR\r\n");
@@ -193,6 +196,7 @@ icalproperty* icalproperty_new_from_string(const char* str)
 
     if(comp == 0){
         icalerror_set_errno(ICAL_PARSE_ERROR);
+        free(buf);
         return 0;
     }
 
@@ -317,7 +321,7 @@ static char*
 fold_property_line (char *text)
 {
     size_t buf_size;
-    char *buf, *buf_ptr, *line_start, *next_line_start, *out_buf;
+    char *buf, *buf_ptr, *line_start, *next_line_start;
     int len, chars_left, first_line;
     char ch;
 
@@ -358,12 +362,7 @@ fold_property_line (char *text)
 	line_start = next_line_start;
     }
 
-    /* Copy it to a temporary buffer, and then free it. */
-    out_buf = icalmemory_tmp_buffer (strlen (buf) + 1);
-    strcpy (out_buf, buf);
-    icalmemory_free_buffer (buf);
-
-    return out_buf;
+    return buf;
 }
 
 
@@ -418,28 +417,37 @@ icalproperty_get_value_kind(icalproperty *prop)
 
 const char*
 icalproperty_as_ical_string (icalproperty* prop)
+{
+	char *buf;
+	buf = icalproperty_as_ical_string_r(prop);
+	icalmemory_add_tmp_buffer(buf);
+	return buf;
+}
+
+   
+char*
+icalproperty_as_ical_string_r(icalproperty* prop)
 {   
     icalparameter *param;
 
     /* Create new buffer that we can append names, parameters and a
-       value to, and reallocate as needed. Later, this buffer will be
-       copied to a icalmemory_tmp_buffer, which is managed internally
-       by libical, so it can be given to the caller without fear of
-       the caller forgetting to free it */
+     * value to, and reallocate as needed.
+     */
 
     const char* property_name = 0; 
     size_t buf_size = 1024;
-    char* buf = icalmemory_new_buffer(buf_size);
-    char* buf_ptr = buf;
+    char* buf;
+    char* buf_ptr;
     icalvalue* value;
     char *out_buf;
     const char* kind_string = 0;
-
-    char newline[] = "\r\n";
+    const char newline[] = "\r\n";
 
     
     icalerror_check_arg_rz( (prop!=0),"prop");
 
+    buf = icalmemory_new_buffer(buf_size);
+    buf_ptr = buf;
 
     /* Append property name */
 
@@ -470,11 +478,7 @@ icalproperty_as_ical_string (icalproperty* prop)
 	param = icalproperty_get_next_parameter(prop,ICAL_ANY_PARAMETER)) {
 
 	icalparameter_kind kind = icalparameter_isa(param);
-	kind_string = icalparameter_as_ical_string(param); 
-
-	if(kind==ICAL_VALUE_PARAMETER){
-	    continue;
-	}
+	kind_string = icalparameter_as_ical_string_r(param); 
 
 	if (kind_string == 0 ) {
 	  icalerror_warn("Got a parameter of unknown kind for the following property");
@@ -483,8 +487,14 @@ icalproperty_as_ical_string (icalproperty* prop)
 	    continue;
 	}
 
+	if (kind==ICAL_VALUE_PARAMETER) {
+		free ((char *) kind_string);
+		continue;
+	}
+
 	icalmemory_append_string(&buf, &buf_ptr, &buf_size, ";");
     	icalmemory_append_string(&buf, &buf_ptr, &buf_size, kind_string);
+	free((char *)kind_string);
     }    
 
     /* Append value */
@@ -494,9 +504,12 @@ icalproperty_as_ical_string (icalproperty* prop)
     value = icalproperty_get_value(prop);
 
     if (value != 0){
-	const char *str = icalvalue_as_ical_string(value);
-	icalerror_assert((str !=0),"Could not get string representation of a value");
-	icalmemory_append_string(&buf, &buf_ptr, &buf_size, str);
+	char *str = icalvalue_as_ical_string_r(value);
+	if (str != 0)
+	    icalmemory_append_string(&buf, &buf_ptr, &buf_size, str);
+	else
+	    icalmemory_append_string(&buf, &buf_ptr, &buf_size,"ERROR: No Value"); 
+	free(str);
     } else {
 	icalmemory_append_string(&buf, &buf_ptr, &buf_size,"ERROR: No Value"); 
 	
@@ -504,9 +517,6 @@ icalproperty_as_ical_string (icalproperty* prop)
     
     icalmemory_append_string(&buf, &buf_ptr, &buf_size, newline);
 
-    /* Now, copy the buffer to a tmp_buffer, which is safe to give to
-       the caller without worring about de-allocating it. */
-
     /* We now use a function to fold the line properly every 75 characters.
        That function also adds the newline for us. */
     out_buf = fold_property_line (buf);
@@ -561,11 +571,15 @@ icalproperty_set_parameter (icalproperty* prop,icalparameter* parameter)
     icalerror_check_arg_rv( (parameter!=0),"parameter");
 
     kind = icalparameter_isa(parameter);
-    if (kind != ICAL_X_PARAMETER)
-      icalproperty_remove_parameter_by_kind(prop,kind);
-    else
+    if (kind == ICAL_X_PARAMETER) {
       icalproperty_remove_parameter_by_name(prop, 
 					    icalparameter_get_xname(parameter));
+    } else if (kind == ICAL_IANA_PARAMETER) {
+      icalproperty_remove_parameter_by_name(prop, 
+					    icalparameter_get_iana_name(parameter));
+    }
+    else
+      icalproperty_remove_parameter_by_kind(prop,kind);
 
     icalproperty_add_parameter(prop,parameter);
 }
@@ -595,8 +609,10 @@ void icalproperty_set_parameter_from_string(icalproperty* prop,
         return;
     }
 
-    if(kind == ICAL_X_PARAMETER){
-	icalparameter_set_xname(param, name);
+    if (kind == ICAL_X_PARAMETER) {
+        icalparameter_set_xname(param, name);
+    } else if (kind == ICAL_IANA_PARAMETER) {
+        icalparameter_set_iana_name(param, name);
     }
 
     icalproperty_set_parameter(prop,param);
@@ -605,11 +621,21 @@ void icalproperty_set_parameter_from_string(icalproperty* prop,
 
 const char* icalproperty_get_parameter_as_string(icalproperty* prop,
                                                  const char* name)
+{
+	char *buf;
+	buf = icalproperty_get_parameter_as_string_r(prop, name);
+	icalmemory_add_tmp_buffer(buf);
+	return buf;
+}
+
+
+char* icalproperty_get_parameter_as_string_r(icalproperty* prop,
+                                                 const char* name)
 {
     icalparameter_kind kind;
     icalparameter *param;
     char* str;
-    char* pv;
+    char *pv, *t;
     char* pvql;
     char* pvqr;
 
@@ -626,13 +652,19 @@ const char* icalproperty_get_parameter_as_string(icalproperty* prop,
     for(param = icalproperty_get_first_parameter(prop,kind); 
 	    param != 0; 
 	    param = icalproperty_get_next_parameter(prop,kind)) {
-	    if (kind != ICAL_X_PARAMETER) {
-		    break;
-	    }
 
-	    if (strcmp(icalparameter_get_xname(param),name)==0) {
+	    if (kind == ICAL_X_PARAMETER) {
+            if (strcmp(icalparameter_get_xname(param),name)==0) {
+                break;
+            }		
+        } else if (kind == ICAL_IANA_PARAMETER) {
+            if (strcmp(icalparameter_get_iana_name(param),name)==0) {
+                break;
+            }		
+	    } else {
 		    break;
-	    }		
+        }
+
     }
 
     if (param == 0){
@@ -640,30 +672,40 @@ const char* icalproperty_get_parameter_as_string(icalproperty* prop,
     }
 
 
-    str = icalparameter_as_ical_string(param);
+    str = icalparameter_as_ical_string_r(param);
 
-    pv = strchr(str,'=');
+    t = strchr(str,'=');
 
-    if(pv == 0){
+    if (t == 0) {
         icalerror_set_errno(ICAL_INTERNAL_ERROR);
+	free(str);
         return 0;
     }
 
-    /*
-     * see if this string is quoted, immediately return if not
-     * otherwise removed the quotes from the string.
-     */
-    ++pv;
-    pvql = strchr(pv,'"');
-    if(pvql == 0)
-        return pv;
-    pvqr = strrchr(pvql,'"');
-    if(pvqr == 0){
+    /* Strip the property name and the equal sign */
+    pv = icalmemory_strdup(t+1);
+    free(str);
+
+    /* Is the string quoted? */
+    pvql = strchr(pv, '"');
+    if (pvql == 0) {
+	return(pv);		/* No quotes?  Return it immediately. */
+    }
+
+    /* Strip everything up to the first quote */
+    str = icalmemory_strdup(pvql+1);
+    free(pv);
+
+    /* Search for the end quote */	
+    pvqr = strrchr(str, '"');
+    if (pvqr == 0) {
         icalerror_set_errno(ICAL_INTERNAL_ERROR);
+	free(str);
         return 0;
     }
+
     *pvqr = '\0';
-    return pvql+1;
+    return str;
 }
 
 /** @see icalproperty_remove_parameter_by_kind() 
@@ -735,6 +777,8 @@ icalproperty_remove_parameter_by_name(icalproperty* prop, const char *name)
 
 	if (icalparameter_isa(param) == ICAL_X_PARAMETER)
 	  kind_string = icalparameter_get_xname(param);
+    else if (icalparameter_isa(param) == ICAL_IANA_PARAMETER)
+	  kind_string = icalparameter_get_iana_name(param);
 	else
 	  kind_string = icalparameter_kind_to_string(icalparameter_isa(param));
 
@@ -743,6 +787,7 @@ icalproperty_remove_parameter_by_name(icalproperty* prop, const char *name)
 
         if (0 == strcmp(kind_string, name)) {
             pvl_remove (prop->parameters, p);
+	    icalparameter_free(param);
             break;
         }
     }                       
@@ -769,21 +814,15 @@ icalproperty_remove_parameter_by_ref(icalproperty* prop, icalparameter* paramete
     icalerror_check_arg_rv((parameter!=0),"parameter");
 
     kind = icalparameter_isa(parameter);
-    name = icalparameter_get_xname(parameter);
 
-    /*
-     * FIXME If it's an X- parameter, also compare the names. It would be nice
-     * to have a better abstraction like icalparameter_equals()
-     */
-    for(p=pvl_head(prop->parameters);p != 0; p = pvl_next(p)){
-	icalparameter* p_param = (icalparameter *)pvl_data (p);
-	if (icalparameter_isa(p_param) == kind &&
-	    (kind != ICAL_X_PARAMETER ||
-	    !strcmp(icalparameter_get_xname(p_param), name))) {
+    for (p=pvl_head(prop->parameters);p != 0; p = pvl_next(p)) {
+        icalparameter* p_param = (icalparameter *)pvl_data (p);
+
+        if (icalparameter_has_same_name(parameter, p_param)) {
             pvl_remove (prop->parameters, p);
             icalparameter_free(p_param);
             break;
-	} 
+        }
     }   
 }
 
@@ -898,6 +937,7 @@ void icalproperty_set_value_from_string(icalproperty* prop,const char* str,
         return;
     }
 
+    icalerror_clear_errno();
     nval = icalvalue_new_from_string(kind, str);
 
     if(nval == 0){
@@ -920,6 +960,15 @@ icalproperty_get_value(const icalproperty* prop)
 }
 
 const char* icalproperty_get_value_as_string(const icalproperty* prop)
+{
+	char *buf;
+	buf = icalproperty_get_value_as_string_r(prop);
+	icalmemory_add_tmp_buffer(buf);
+	return buf;
+}
+
+
+char* icalproperty_get_value_as_string_r(const icalproperty* prop)
 {
     icalvalue *value;
     
@@ -927,7 +976,7 @@ const char* icalproperty_get_value_as_string(const icalproperty* prop)
 
     value = prop->value; 
 
-    return icalvalue_as_ical_string(value);
+    return icalvalue_as_ical_string_r(value);
 }
 
 
@@ -955,14 +1004,26 @@ const char* icalproperty_get_x_name(icalproperty* prop){
 }
 
 const char* icalproperty_get_property_name(const icalproperty* prop)
+{
+	char *buf;
+	buf = icalproperty_get_property_name_r(prop);
+	icalmemory_add_tmp_buffer(buf);
+	return buf;
+}
+
+
+char* icalproperty_get_property_name_r(const icalproperty* prop)
 {
 
     const char* property_name = 0;
     size_t buf_size = 256;
-    char* buf = icalmemory_new_buffer(buf_size);
-    char* buf_ptr = buf;  
+    char* buf;
+    char* buf_ptr;
 
     icalerror_check_arg_rz( (prop!=0),"prop");
+
+    buf = icalmemory_new_buffer(buf_size);
+    buf_ptr = buf;
  
     if (prop->kind == ICAL_X_PROPERTY && prop->x_name != 0){
         property_name = prop->x_name;
@@ -972,6 +1033,7 @@ const char* icalproperty_get_property_name(const icalproperty* prop)
  
     if (property_name == 0 ) {
         icalerror_set_errno(ICAL_MALFORMEDDATA_ERROR);
+	icalmemory_free_buffer(buf);
         return 0;
 
     } else {
@@ -980,10 +1042,6 @@ const char* icalproperty_get_property_name(const icalproperty* prop)
         icalmemory_append_string(&buf, &buf_ptr, &buf_size, property_name);
     }
  
-    /* Add the buffer to the temporary buffer ring -- the caller will
-       not have to free the memory. */
-    icalmemory_add_tmp_buffer(buf);
- 
     return buf;
 }
                             
