@@ -329,6 +329,7 @@ char* set_tz(const char* tzid)
 
     if(new_tz == 0){
 	icalerror_set_errno(ICAL_NEWFAILED_ERROR);
+	free(old_tz_copy);
 	return 0;
     }
 
@@ -362,8 +363,12 @@ void unset_tz(char *tzstr)
 	   calls to time related functions in libc will treat times in UTC. */
 #ifdef HAVE_UNSETENV
 	unsetenv("TZ");
+#else
+#ifdef _MSC_VER 
+	putenv("TZ="); // The equals is required to remove with MS Visual C++
 #else
 	putenv("TZ");
+#endif
 #endif
     } 
 
@@ -426,15 +431,11 @@ to prevent any crashes */
     
     /* Set TZ to UTC and use mktime to convert to a time_t. */
     old_tz = set_tz ("UTC");
-#ifdef WIN32
     tzset ();
-#endif
 
     t = mktime (&stm);
     unset_tz (old_tz);
-#ifdef WIN32
     tzset ();
-#endif
 
 #ifdef HAVE_PTHREAD
     pthread_mutex_unlock (&tzid_mutex);
@@ -442,11 +443,20 @@ to prevent any crashes */
     return t;
 }
 
+const char* icaltime_as_ical_string(const struct icaltimetype tt)
+{
+	char *buf;
+	buf = icaltime_as_ical_string_r(tt);
+	icalmemory_add_tmp_buffer(buf);
+	return buf;
+}
+
+
 /**
  * Return a string represention of the time, in RFC2445 format. The
  * string is owned by libical
  */
-const char* icaltime_as_ical_string(const struct icaltimetype tt)
+char* icaltime_as_ical_string_r(const struct icaltimetype tt)
 {
     size_t size = 17;
     char* buf = icalmemory_new_buffer(size);
@@ -464,12 +474,10 @@ const char* icaltime_as_ical_string(const struct icaltimetype tt)
 		 tt.hour,tt.minute,tt.second);
     }
     
-    icalmemory_add_tmp_buffer(buf);
-
     return buf;
-
 }
 
+
 /**
  *	Reset all of the time components to be in their normal ranges. For
  *	instance, given a time with minutes=70, the minutes will be reduces
@@ -590,8 +598,17 @@ int icaltime_days_in_month(const int month, const int year)
 
     int days = _days_in_month[month];
 
-    assert(month > 0);
-    assert(month <= 12);
+/* The old code aborting if it was passed a parameter like BYMONTH=0
+ * Unfortunately it's not practical right now to pass an error all
+ * the way up the stack, so instead of aborting we're going to apply
+ * the GIGO principle and simply return '30 days' if we get an
+ * invalid month.  Modern applications cannot tolerate crashing.
+ *  assert(month > 0);
+ *  assert(month <= 12);
+ */
+    if ((month < 1) || (month > 12)) {
+	return 30;
+    }
 
     if( month == 2){
 	days += icaltime_is_leap_year(year);
