@@ -40,12 +40,20 @@
 #include "icalerror.h"
 #include "icalparser.h"
 #include "icaltimezone.h"
+#include "icaltimezoneimpl.h"
 #include "icaltz-util.h"
 
 #include <sys/stat.h>
 
+#ifdef HAVE_PTHREAD
+#include <pthread.h>
+static pthread_mutex_t builtin_mutex = PTHREAD_MUTEX_INITIALIZER;
+#endif
+
 #ifdef WIN32
+#ifndef _WIN32_WCE
 #include <mbstring.h>
+#endif
 #include <windows.h>
 /* Undef the similar macro from pthread.h, it doesn't check if
  * gmtime() returns NULL.
@@ -54,14 +62,25 @@
 
 /* The gmtime() in Microsoft's C library is MT-safe */
 #define gmtime_r(tp,tmp) (gmtime(tp)?(*(tmp)=*gmtime(tp),(tmp)):0)
+
+// MSVC lacks the POSIX macro S_ISDIR, however it's a trivial one:
+#ifndef S_ISDIR
+#define S_ISDIR(m) (((m) & _S_IFMT) == _S_IFDIR)
+#ifndef HAVE_SNPRINTF
+#include "vsnprintf.h"
+#endif
+#endif
+
+#define strcasecmp      stricmp
 #endif
 
 /** This is the toplevel directory where the timezone data is installed in. */
 #define ZONEINFO_DIRECTORY	PACKAGE_DATA_DIR "/zoneinfo"
 
-/** The prefix we use to uniquely identify TZIDs. */
-#define TZID_PREFIX		"/citadel.org/"
-#define TZID_PREFIX_LEN		13
+/** The prefix we use to uniquely identify TZIDs.
+    It must begin and end with forward slashes.
+ */
+const char *ical_tzid_prefix =	"/freeassociation.sourceforge.net/";
 
 /** This is the filename of the file containing the city names and
     coordinates of all the builtin timezones. */
@@ -75,54 +94,6 @@
     somewhere around 2037. */
 #define ICALTIMEZONE_MAX_YEAR		2035
 
-struct _icaltimezone {
-    char		*tzid;
-    /**< The unique ID of this timezone,
-       e.g. "/citadel.org/Olson_20010601_1/Africa/Banjul".
-       This should only be used to identify a VTIMEZONE. It is not
-       meant to be displayed to the user in any form. */
-
-    char		*location;
-    /**< The location for the timezone, e.g. "Africa/Accra" for the
-       Olson database. We look for this in the "LOCATION" or
-       "X-LIC-LOCATION" properties of the VTIMEZONE component. It
-       isn't a standard property yet. This will be NULL if no location
-       is found in the VTIMEZONE. */
-
-    char		*tznames;
-    /**< This will be set to a combination of the TZNAME properties
-       from the last STANDARD and DAYLIGHT components in the
-       VTIMEZONE, e.g. "EST/EDT".  If they both use the same TZNAME,
-       or only one type of component is found, then only one TZNAME
-       will appear, e.g. "AZOT". If no TZNAME is found this will be
-       NULL. */
-
-    double		 latitude;
-    double		 longitude;
-    /**< The coordinates of the city, in degrees. */
-
-    icalcomponent	*component;
-    /**< The toplevel VTIMEZONE component loaded from the .ics file for this
-         timezone. If we need to regenerate the changes data we need this. */
-
-    icaltimezone	*builtin_timezone;
-    /**< If this is not NULL it points to the builtin icaltimezone
-       that the above TZID refers to. This icaltimezone should be used
-       instead when accessing the timezone changes data, so that the
-       expanded timezone changes data is shared between calendar
-       components. */
-
-    int			 end_year;
-    /**< This is the last year for which we have expanded the data to.
-       If we need to calculate a date past this we need to expand the
-       timezone component data from scratch. */
-
-    icalarray		*changes;
-    /**< A dynamically-allocated array of time zone changes, sorted by the
-       time of the change in local time. So we can do fast binary-searches
-       to convert from local time to UTC. */
-};
-
 typedef struct _icaltimezonechange	icaltimezonechange;
 
 struct _icaltimezonechange {
@@ -233,10 +204,18 @@ icaltimezone_copy			(icaltimezone *originalzone)
     }
 
     memcpy (zone, originalzone, sizeof (icaltimezone));
+    if (zone->tzid != NULL) 
+	zone->tzid = strdup (zone->tzid);
     if (zone->location != NULL) 
 	zone->location = strdup (zone->location);
     if (zone->tznames != NULL)
 	zone->tznames = strdup (zone->tznames);
+    if (zone->changes != NULL)
+        zone->changes = icalarray_copy(zone->changes);
+    
+    /* Let the caller set the component because then they will
+       know to be careful not to free this reference twice. */
+    zone->component = NULL;
 
     return zone;
 }
@@ -308,17 +287,16 @@ icaltimezone_get_vtimezone_properties	(icaltimezone *zone,
     if (!tzid)
 	return 0;
 
-    prop = icalcomponent_get_first_property (component, ICAL_TZNAME_PROPERTY);
-    if (prop) {
-	tzname = icalproperty_get_tzname (prop);
-	zone->tznames = strdup(tzname);	
-    } else
-	zone->tznames = NULL;
-    
+    if (zone->tzid) free(zone->tzid);
     zone->tzid = strdup (tzid);
+
+    if (zone->component) icalcomponent_free(zone->component);
     zone->component = component;
-	if ( zone->location != 0 ) free ( zone->location );
+
+    if (zone->location) free(zone->location);
     zone->location = icaltimezone_get_location_from_vtimezone (component);
+
+    if (zone->tznames) free(zone->tznames);
     zone->tznames = icaltimezone_get_tznames_from_vtimezone (component);
 
     return 1;
@@ -481,8 +459,7 @@ icaltimezone_ensure_coverage		(icaltimezone *zone,
 
     int changes_end_year;
 
-    if (!zone->component)
-	icaltimezone_load_builtin_timezone (zone);
+    icaltimezone_load_builtin_timezone (zone);
 
     if (icaltimezone_minimum_expansion_year == -1) {
 	struct icaltimetype today = icaltime_today();
@@ -688,7 +665,7 @@ icaltimezone_expand_vtimezone		(icalcomponent	*comp,
 	    }
 
 	    rrule_iterator = icalrecur_iterator_new (rrule, dtstart);
-	    for (;;) {
+	    for (;rrule_iterator;) {
 		occ = icalrecur_iterator_next (rrule_iterator);
 		if (occ.year > end_year || icaltime_is_null_time (occ))
 		    break;
@@ -1168,8 +1145,7 @@ icaltimezone_get_tzid			(icaltimezone *zone)
     if (!zone)
 	return NULL;
 
-    if (!zone->tzid)
-	icaltimezone_load_builtin_timezone (zone);
+    icaltimezone_load_builtin_timezone (zone);
 
     return zone->tzid;
 }
@@ -1195,8 +1171,7 @@ icaltimezone_get_tznames		(icaltimezone *zone)
     if (!zone)
 	return NULL;
 
-    if (!zone->component)
-	icaltimezone_load_builtin_timezone (zone);
+    icaltimezone_load_builtin_timezone (zone);
 
     return zone->tznames;
 }
@@ -1238,8 +1213,7 @@ icaltimezone_get_component		(icaltimezone *zone)
     if (!zone)
 	return NULL;
 
-    if (!zone->component)
-	icaltimezone_load_builtin_timezone (zone);
+    icaltimezone_load_builtin_timezone (zone);
 
     return zone->component;
 }
@@ -1276,7 +1250,7 @@ icaltimezone_get_display_name		(icaltimezone	*zone)
 		   this is one of our TZIDs and if so we jump to the city name
 		   at the end of it. */
 		if (display_name
-		    && !strncmp (display_name, TZID_PREFIX, TZID_PREFIX_LEN)) {
+		    && !strncmp (display_name, ical_tzid_prefix, strlen(ical_tzid_prefix))) {
 		    /* Get the location, which is after the 3rd '/' char. */
 		    const char *p;
 		    int num_slashes = 0;
@@ -1293,7 +1267,6 @@ icaltimezone_get_display_name		(icaltimezone	*zone)
 	return display_name;
 }
 
-
 icalarray*
 icaltimezone_array_new			(void)
 {
@@ -1354,6 +1327,7 @@ void
 icaltimezone_free_builtin_timezones(void)
 {
 	icaltimezone_array_free(builtin_timezones);
+	builtin_timezones = 0;
 }
 
 
@@ -1361,6 +1335,7 @@ icaltimezone_free_builtin_timezones(void)
 icaltimezone*
 icaltimezone_get_builtin_timezone	(const char *location)
 {
+    icalcomponent *comp;
     icaltimezone *zone;
     int lower;
     const char *zone_location;
@@ -1402,6 +1377,20 @@ icaltimezone_get_builtin_timezone	(const char *location)
 		return zone;
     }
 
+    /* Check whether file exists, but is not mentioned in zone.tab.
+       It means it's a deprecated timezone, but still available. */
+    comp = icaltzutil_fetch_timezone (location);
+    if (comp) {
+	icaltimezone tz;
+	icaltimezone_init (&tz);
+	if (icaltimezone_set_component (&tz, comp)) {
+	    icalarray_append (builtin_timezones, &tz);
+	    return icalarray_element_at (builtin_timezones, builtin_timezones->num_elements - 1);
+	} else {
+	    icalcomponent_free (comp);
+	}
+    }
+
     return NULL;
 }
 
@@ -1462,8 +1451,7 @@ icaltimezone_get_builtin_timezone_from_offset	(int offset, const char *tzname)
     for (i=0; i<count; i++) {
 	int z_offset;
 	zone = icalarray_element_at (builtin_timezones, i);
-	if (!zone->component)
-	    icaltimezone_load_builtin_timezone (zone);
+        icaltimezone_load_builtin_timezone (zone);
 	
 	z_offset = get_offset(zone);
 
@@ -1486,20 +1474,21 @@ icaltimezone_get_builtin_timezone_from_tzid (const char *tzid)
 	return NULL;
 
     /* Check that the TZID starts with our unique prefix. */
-    if (strncmp (tzid, TZID_PREFIX, TZID_PREFIX_LEN))
+    if (strncmp (tzid, ical_tzid_prefix, strlen(ical_tzid_prefix)))
 	return NULL;
 
-    /* Get the location, which is after the 3rd '/' character. */
+    /* Get the location, which is after the 4th '/' character. */
+    /* format is /unique_prefix/Olson_X/Tzfile/location */
     p = tzid;
     for (p = tzid; *p; p++) {
 	if (*p == '/') {
 	    num_slashes++;
-	    if (num_slashes == 3)
+	    if (num_slashes == 4)
 		break;
 	}
     }
 
-    if (num_slashes != 3)
+    if (num_slashes != 4)
 	return NULL;
 
     p++;
@@ -1541,7 +1530,14 @@ icaltimezone_init_builtin_timezones	(void)
     /* Initialize the special UTC timezone. */
     utc_timezone.tzid = (char *)"UTC";
 
-    icaltimezone_parse_zone_tab ();
+#ifdef HAVE_PTHREAD
+    pthread_mutex_lock(&builtin_mutex);
+#endif
+    if (!builtin_timezones)
+        icaltimezone_parse_zone_tab ();
+#ifdef HAVE_PTHREAD
+    pthread_mutex_unlock(&builtin_mutex);
+#endif
 }
 
 static int
@@ -1616,8 +1612,10 @@ fetch_lat_long_from_string  (const char *str, int *latitude_degrees, int *latitu
 
 	if (parse_coord (lat, lon - lat, latitude_degrees, latitude_minutes, latitude_seconds) == 1 ||
 		       	parse_coord (lon, strlen (lon), longitude_degrees, longitude_minutes, longitude_seconds) 
-			== 1)
-			return 1;
+			== 1) {
+				free(lat);
+				return 1;
+			}
 	
 	free (lat);
 
@@ -1756,6 +1754,12 @@ icaltimezone_load_builtin_timezone	(icaltimezone *zone)
     if (!zone->location || !zone->location[0])
 	return;
 
+#ifdef HAVE_PTHREAD
+    pthread_mutex_lock(&builtin_mutex);
+    if (zone->component)
+       goto out;
+#endif
+
 #ifdef USE_BUILTIN_TZDATA
     {
     char *filename;
@@ -1812,8 +1816,11 @@ icaltimezone_load_builtin_timezone	(icaltimezone *zone)
     icalcomponent_remove_component(comp,subcomp);
     icalcomponent_free(comp);
     }
-#endif    
-
+#endif 
+#ifdef HAVE_PTHREAD
+ out:
+    pthread_mutex_unlock(&builtin_mutex);
+#endif
 }
 
 
@@ -1929,10 +1936,18 @@ static const char* get_zone_directory(void)
 	return zone_files_directory == NULL ? ZONEINFO_DIRECTORY : zone_files_directory;
 #else
 	wchar_t wbuffer[1000];
+#ifndef _WIN32_WCE
 	char buffer[1000], zoneinfodir[1000], dirname[1000];
+#else
+	wchar_t zoneinfodir[1000], dirname[1000];
+#endif
 	int used_default;
 	static char *cache = NULL;
+#ifndef _WIN32_WCE
 	char *dirslash, *zislash;
+#else
+	wchar_t *dirslash, *zislash;
+#endif
 	struct stat st;
 
 	if (zone_files_directory)
@@ -1945,6 +1960,8 @@ static const char* get_zone_directory(void)
 	if (!GetModuleFileNameW (NULL, wbuffer, sizeof (wbuffer) / sizeof (wbuffer[0])))
 	    return ZONEINFO_DIRECTORY;
 
+/*wince supports only unicode*/
+#ifndef _WIN32_WCE
 	/* Convert to system codepage */
 	if (!WideCharToMultiByte (CP_ACP, 0, wbuffer, -1, buffer, sizeof (buffer),
 				  NULL, &used_default) ||
@@ -1957,6 +1974,7 @@ static const char* get_zone_directory(void)
 		used_default)
 		return ZONEINFO_DIRECTORY;
 	}
+#endif
 	/* Look for the zoneinfo directory somewhere in the path where
 	 * the app is installed. If the path to the app is
 	 *
@@ -1992,10 +2010,38 @@ static const char* get_zone_directory(void)
 	 */
 
 	/* Strip away basename of app .exe first */
+#ifndef _WIN32_WCE
 	dirslash = _mbsrchr (buffer, '\\');
+#else
+	dirslash = wcsrchr (wbuffer, L'\\');
+#endif
 	if (dirslash)
+#ifndef _WIN32_WCE
 	    *dirslash = '\0';
+#else
+	    *dirslash = L'\0';
+#endif
 
+#ifdef _WIN32_WCE
+	while ((dirslash = wcsrchr (wbuffer, '\\'))) {
+	    /* Strip one more directory from app .exe location */
+	    *dirslash = L'\0';
+	    
+        MultiByteToWideChar(CP_ACP,0,ZONEINFO_DIRECTORY,-1,zoneinfodir,1000);
+        
+	    while ((zislash = wcschr (zoneinfodir, L'/'))) {
+		*zislash = L'.';
+		wcscpy (dirname, wbuffer);
+		wcscat (dirname, "/");
+		wcscat (dirname, zislash + 1);
+		if (stat (dirname, &st) == 0 &&
+		    S_ISDIR (st.st_mode)) {
+		    cache = wce_wctomb (dirname);
+		    return cache;
+		}
+	    }
+	}
+#else
 	while ((dirslash = _mbsrchr (buffer, '\\'))) {
 	    /* Strip one more directory from app .exe location */
 	    *dirslash = '\0';
@@ -2013,12 +2059,15 @@ static const char* get_zone_directory(void)
 		}
 	    }
 	}
+#endif
 	return ZONEINFO_DIRECTORY;
 #endif
 }
 
 void set_zone_directory(char *path)
 {
+	if (zone_files_directory)
+		free_zone_directory();
 	zone_files_directory = malloc(strlen(path)+1);
 	if ( zone_files_directory != NULL )
 	{
@@ -2031,5 +2080,13 @@ void free_zone_directory(void)
 	if ( zone_files_directory != NULL )
 	{
 		free(zone_files_directory);
+		zone_files_directory = NULL;
+	}
+}
+
+void icaltimezone_set_tzid_prefix(const char *new_prefix)
+{
+	if (new_prefix) {
+		ical_tzid_prefix = new_prefix;
 	}
 }
