@@ -136,6 +136,7 @@ icalvalue* icalvalue_new_clone(const icalvalue* old) {
 		new->data.v_string=icalmemory_strdup(old->data.v_string);
 
 		if ( new->data.v_string == 0 ) {
+                    icalvalue_free(new);
 		    return 0;
 		}		    
 
@@ -148,6 +149,7 @@ icalvalue* icalvalue_new_clone(const icalvalue* old) {
 		new->data.v_recur = malloc(sizeof(struct icalrecurrencetype));
 
 		if(new->data.v_recur == 0){
+                    icalvalue_free(new);
 		    return 0;
 		}
 
@@ -163,6 +165,7 @@ icalvalue* icalvalue_new_clone(const icalvalue* old) {
 		new->x_value=icalmemory_strdup(old->x_value);
 
 		if (new->x_value == 0) {
+                    icalvalue_free(new);
 		    return 0;
 		}
 	    }
@@ -262,6 +265,84 @@ static char* icalmemory_strdup_and_dequote(const char* str)
     return out;
 }
 
+/*
+ * Returns a quoted copy of a string
+ * @todo This is not RFC2445 compliant.
+ * The RFC only allows:
+ * TSAFE-CHAR = %x20-21 / %x23-2B / %x2D-39 / %x3C-5B / %x5D-7E / NON-US-ASCII
+ * As such, \t\r\b\f are not allowed, not even escaped
+ */
+static char* icalmemory_strdup_and_quote(const icalvalue* value, 
+                                         const char* unquoted_str)
+{
+    char *str;
+    char *str_p;
+    const char *p;
+    size_t buf_sz;
+
+    buf_sz = strlen(unquoted_str)+1;
+
+    str_p = str = (char*)icalmemory_new_buffer(buf_sz);
+
+    if (str_p == 0){
+      return 0;
+    }
+
+    for(p=unquoted_str; *p!=0; p++){
+
+	switch(*p){
+	    case '\n': {
+            icalmemory_append_string(&str,&str_p,&buf_sz,"\\n");
+            break;
+	    }
+
+	    case '\t': {
+            icalmemory_append_string(&str,&str_p,&buf_sz,"\\t");
+            break;
+	    }
+	    case '\r': {
+            icalmemory_append_string(&str,&str_p,&buf_sz,"\\r");
+            break;
+	    }
+	    case '\b': {
+            icalmemory_append_string(&str,&str_p,&buf_sz,"\\b");
+            break;
+	    }
+	    case '\f': {
+            icalmemory_append_string(&str,&str_p,&buf_sz,"\\f");
+            break;
+	    }
+
+	    case ';':
+        case ',':
+            /* unescaped COMMA is allowed in CATEGORIES property as its
+               considered a list delimiter here, see:
+               http://tools.ietf.org/html/rfc2445#section-4.3.11 */
+            if (icalproperty_isa(value->parent) == ICAL_CATEGORIES_PROPERTY) {
+                icalmemory_append_char(&str,&str_p,&buf_sz,*p);
+                break;
+            }
+	    case '"':
+	    case '\\': {
+            icalmemory_append_char(&str,&str_p,&buf_sz,'\\');
+            icalmemory_append_char(&str,&str_p,&buf_sz,*p);
+            break;
+	    }
+
+	    default: {
+            icalmemory_append_char(&str,&str_p,&buf_sz,*p);
+	    }
+	}
+    }
+
+    /* Assume the last character is not a '\0' and add one. We could
+       check *str_p != 0, but that would be an uninitialized memory
+       read. */
+    
+    icalmemory_append_char(&str,&str_p,&buf_sz,'\0');
+    return str;
+}
+
 /*
  * FIXME
  *
@@ -297,14 +378,16 @@ icalvalue* icalvalue_new_enum(icalvalue_kind kind, int x_type, const char* str)
  * If you want a code that that does the same job with a decimal separator
  * dependant on the current locale, then use strtof() from libc.
  */
-int simple_str_to_float(const char* from,
-                        float *result,
-                        char** to)
+int simple_str_to_double(const char* from,
+                         double *result,
+                         char** to)
 {
 #define TMP_NUM_SIZE 100
     char *start=NULL, *end=NULL, *cur=(char*)from ;
     char tmp_buf[TMP_NUM_SIZE+1] ; /*hack*/
+#ifndef _WIN32_WCE
     struct lconv *loc_data = localeconv () ;
+#endif
     int i=0 ;
 
     /*sanity checks*/
@@ -341,6 +424,7 @@ int simple_str_to_float(const char* from,
      * care to have the (optional) decimal separator be the one
      * of the current locale.
      */
+#ifndef _WIN32_WCE
     for (i=0 ; i < end - from ;++i) {
         if (start[i] == '.'
             && loc_data
@@ -353,6 +437,9 @@ int simple_str_to_float(const char* from,
             tmp_buf[i] = start[i] ;
         }
     }
+#else
+    GetNumberFormat(LOCALE_SYSTEM_DEFAULT,0,start, NULL, tmp_buf,TMP_NUM_SIZE);
+#endif
     if (to)
         *to = end ;
     *result = atof(tmp_buf) ;
@@ -388,7 +475,7 @@ icalvalue* icalvalue_new_from_string_with_error(icalvalue_kind kind,const char*
     case ICAL_BINARY_VALUE:
     {
         icalattach *attach;
-        attach = icalattach_new_from_data ((unsigned char*)str, 0, 0);
+        attach = icalattach_new_from_data (str, 0, 0);
         if ( !attach )
           break;
         value = icalvalue_new_attach (attach);
@@ -495,7 +582,7 @@ icalvalue* icalvalue_new_from_string_with_error(icalvalue_kind kind,const char*
         char *cur=NULL ;
         struct icalgeotype geo = {0.0, 0.0};
   
-        if (simple_str_to_float (str, &geo.lat, &cur)) {
+        if (simple_str_to_double (str, &geo.lat, &cur)) {
             goto geo_parsing_error ;
         }
   
@@ -518,7 +605,7 @@ icalvalue* icalvalue_new_from_string_with_error(icalvalue_kind kind,const char*
             ++cur ;
         }
 
-        if (simple_str_to_float (cur, &geo.lon, &cur)) {
+        if (simple_str_to_double (cur, &geo.lon, &cur)) {
             goto geo_parsing_error ;
         }
         value = icalvalue_new_geo (geo) ;
@@ -706,6 +793,7 @@ icalvalue_free (icalvalue* v)
 	case ICAL_TEXT_VALUE:
 	case ICAL_CALADDRESS_VALUE:
 	case ICAL_URI_VALUE:
+	case ICAL_STRING_VALUE:
 	case ICAL_QUERY_VALUE:
 	{
 	    if (v->data.v_string != 0) { 
@@ -747,7 +835,7 @@ icalvalue_is_valid (const icalvalue* value)
     return 1;
 }
 
-static char* icalvalue_binary_as_ical_string(const icalvalue* value) {
+static char* icalvalue_binary_as_ical_string_r(const icalvalue* value) {
 
     const char* data;
     char* str;
@@ -755,7 +843,7 @@ static char* icalvalue_binary_as_ical_string(const icalvalue* value) {
 
     data = icalvalue_get_binary(value);
 
-    str = (char*)icalmemory_tmp_buffer(60);
+    str = (char*)icalmemory_new_buffer(60);
     snprintf(str, 60,"icalvalue_binary_as_ical_string is not implemented yet");
 
     return str;
@@ -764,11 +852,12 @@ static char* icalvalue_binary_as_ical_string(const icalvalue* value) {
 
 #define MAX_INT_DIGITS 12 /* Enough for 2^32 + sign*/ 
     
-static char* icalvalue_int_as_ical_string(const icalvalue* value) {
+static char* icalvalue_int_as_ical_string_r(const icalvalue* value) {
     int data;
-    char* str = (char*)icalmemory_tmp_buffer(MAX_INT_DIGITS); 
+    char* str;
 
     icalerror_check_arg_rz( (value!=0),"value");
+    str = (char*)icalmemory_new_buffer(MAX_INT_DIGITS); 
 
     data = icalvalue_get_integer(value);
 	
@@ -777,14 +866,16 @@ static char* icalvalue_int_as_ical_string(const icalvalue* value) {
     return str;
 }
 
-static char* icalvalue_utcoffset_as_ical_string(const icalvalue* value)
+
+static char* icalvalue_utcoffset_as_ical_string_r(const icalvalue* value)
 {    
     int data,h,m,s;
     char sign;
-    char* str = (char*)icalmemory_tmp_buffer(9);
+    char* str;
 
     icalerror_check_arg_rz( (value!=0),"value");
 
+    str = (char*)icalmemory_new_buffer(9);
     data = icalvalue_get_utcoffset(value);
 
     if (abs(data) == data){
@@ -805,14 +896,14 @@ static char* icalvalue_utcoffset_as_ical_string(const icalvalue* value)
     return str;
 }
 
-static char* icalvalue_string_as_ical_string(const icalvalue* value) {
+static char* icalvalue_string_as_ical_string_r(const icalvalue* value) {
 
     const char* data;
     char* str = 0;
     icalerror_check_arg_rz( (value!=0),"value");
     data = value->data.v_string;
 
-    str = (char*)icalmemory_tmp_buffer(strlen(data)+1);   
+    str = (char*)icalmemory_new_buffer(strlen(data)+1);   
 
     strcpy(str,data);
 
@@ -820,91 +911,20 @@ static char* icalvalue_string_as_ical_string(const icalvalue* value) {
 }
 
 
-static char* icalvalue_recur_as_ical_string(const icalvalue* value) 
+static char* icalvalue_recur_as_ical_string_r(const icalvalue* value) 
 {
     struct icalrecurrencetype *recur = value->data.v_recur;
-
-    return icalrecurrencetype_as_string(recur);
+    return icalrecurrencetype_as_string_r(recur);
 }
 
- /* @todo This is not RFC2445 compliant.
- * The RFC only allows:
- * TSAFE-CHAR = %x20-21 / %x23-2B / %x2D-39 / %x3C-5B / %x5D-7E / NON-US-ASCII
- * As such, \t\r\b\f are not allowed, not even escaped
- */
-
-static char* icalvalue_text_as_ical_string(const icalvalue* value) {
-    char *str;
-    char *str_p;
-    char *rtrn;
-    const char *p;
-    size_t buf_sz;
-
-    buf_sz = strlen(value->data.v_string)+1;
-
-    str_p = str = (char*)icalmemory_new_buffer(buf_sz);
-
-    if (str_p == 0){
-      return 0;
-    }
-
-    for(p=value->data.v_string; *p!=0; p++){
-
-	switch(*p){
-	    case '\n': {
-		icalmemory_append_string(&str,&str_p,&buf_sz,"\\n");
-		break;
-	    }
-
-	    case '\t': {
-		icalmemory_append_string(&str,&str_p,&buf_sz,"\\t");
-		break;
-	    }
-	    case '\r': {
-		icalmemory_append_string(&str,&str_p,&buf_sz,"\\r");
-		break;
-	    }
-	    case '\b': {
-		icalmemory_append_string(&str,&str_p,&buf_sz,"\\b");
-		break;
-	    }
-	    case '\f': {
-		icalmemory_append_string(&str,&str_p,&buf_sz,"\\f");
-		break;
-	    }
-
-	    case ';':
-	    case ',':
-	    case '"':
-	    case '\\':{
-		icalmemory_append_char(&str,&str_p,&buf_sz,'\\');
-		icalmemory_append_char(&str,&str_p,&buf_sz,*p);
-		break;
-	    }
-
-	    default: {
-		icalmemory_append_char(&str,&str_p,&buf_sz,*p);
-	    }
-	}
-    }
-
-    /* Assume the last character is not a '\0' and add one. We could
-       check *str_p != 0, but that would be an uninitialized memory
-       read. */
-
-
-    icalmemory_append_char(&str,&str_p,&buf_sz,'\0');
-
-    rtrn = icalmemory_tmp_copy(str);
-
-    icalmemory_free_buffer(str);
-
-    return rtrn;
+static char* icalvalue_text_as_ical_string_r(const icalvalue* value) {
+    return icalmemory_strdup_and_quote(value, value->data.v_string);
 }
 
 
+
 static char* 
-icalvalue_attach_as_ical_string(const icalvalue* value) 
+icalvalue_attach_as_ical_string_r(const icalvalue* value) 
 {
     icalattach *a;
     char * str;
@@ -917,63 +937,69 @@ icalvalue_attach_as_ical_string(const icalvalue* value)
 	const char *url;
 
 	url = icalattach_get_url (a);
-	str = icalmemory_tmp_buffer (strlen (url) + 1);
+	str = icalmemory_new_buffer (strlen (url) + 1);
 	strcpy (str, url);
 	return str;
     } else {
-/*	return icalvalue_binary_as_ical_string (value);*/
       const char *data = 0;
       data = (const char*)icalattach_get_data(a);
-      str = icalmemory_tmp_buffer (strlen (data) + 1);
+      str = icalmemory_new_buffer (strlen (data) + 1);
       strcpy (str, data);
       return str;
 }
 }
 
 
-static char* icalvalue_duration_as_ical_string(const icalvalue* value) {
+static char* icalvalue_duration_as_ical_string_r(const icalvalue* value) {
 
     struct icaldurationtype data;
 
     icalerror_check_arg_rz( (value!=0),"value");
     data = icalvalue_get_duration(value);
 
-    return icaldurationtype_as_ical_string(data);
+    return icaldurationtype_as_ical_string_r(data);
 }
 
+
+
 void print_time_to_string(char* str, const struct icaltimetype *data)
 {
     char temp[20];
+    str[0] = '\0';
 
-    if (icaltime_is_utc(*data)){
-    snprintf(temp,sizeof(temp),"%02d%02d%02dZ",data->hour,data->minute,data->second);
-    } else {
-    snprintf(temp,sizeof(temp),"%02d%02d%02d",data->hour,data->minute,data->second);
-    }   
-
-    strcat(str,temp);
+    if (data != 0) {
+        if (icaltime_is_utc(*data)){
+            snprintf(temp,sizeof(temp),"%02d%02d%02dZ",data->hour,data->minute,data->second);
+            strncat(str,temp,7);
+        } else {
+            snprintf(temp,sizeof(temp),"%02d%02d%02d",data->hour,data->minute,data->second);
+            strncat(str,temp,6);
+        }   
+    }
 }
 
  
 void print_date_to_string(char* str,  const struct icaltimetype *data)
 {
     char temp[20];
+    str[0] = '\0';
 
-    snprintf(temp,sizeof(temp),"%04d%02d%02d",data->year,data->month,data->day);
-
-    strcat(str,temp);
+    if (data != 0) {
+        snprintf(temp,sizeof(temp),"%04d%02d%02d",data->year,data->month,data->day);
+        strncat(str,temp,8);
+    }
 }
 
-static char* icalvalue_date_as_ical_string(const icalvalue* value) {
+static char* icalvalue_date_as_ical_string_r(const icalvalue* value) {
 
     struct icaltimetype data;
     char* str;
     icalerror_check_arg_rz( (value!=0),"value");
     data = icalvalue_get_date(value);
 
-    str = (char*)icalmemory_tmp_buffer(9);
+    str = (char*)icalmemory_new_buffer(9);
  
-    str[0] = 0;
+    str[0] = '\0';
     print_date_to_string(str,&data);
    
     return str;
@@ -981,14 +1007,22 @@ static char* icalvalue_date_as_ical_string(const icalvalue* value) {
 
 void print_datetime_to_string(char* str,  const struct icaltimetype *data)
 {
-    print_date_to_string(str,data);
-    if ( !data->is_date ) {
-        strcat(str,"T");
-        print_time_to_string(str,data);
+    char temp[20];
+    str[0] = '\0';
+
+    if (data != 0) {
+        print_date_to_string(str,data);
+        if ( !data->is_date ) {
+            strncat(str,"T",19);
+            temp[0] = '\0';
+            print_time_to_string(temp,data);
+            strncat(str,temp,19);
+        }
     }
 }
 
-static const char* icalvalue_datetime_as_ical_string(const icalvalue* value) {
+
+static char* icalvalue_datetime_as_ical_string_r(const icalvalue* value) {
     
     struct icaltimetype data;
     char* str;
@@ -1005,31 +1039,31 @@ static const char* icalvalue_datetime_as_ical_string(const icalvalue* value) {
 
     data = icalvalue_get_datetime(value);
 
-    str = (char*)icalmemory_tmp_buffer(20);
+    str = (char*)icalmemory_new_buffer(20);
  
     str[0] = 0;
-
     print_datetime_to_string(str,&data);
    
     return str;
 
 }
 
-static char* icalvalue_float_as_ical_string(const icalvalue* value) {
+static char* icalvalue_float_as_ical_string_r(const icalvalue* value) {
 
     float data;
     char* str;
     icalerror_check_arg_rz( (value!=0),"value");
     data = icalvalue_get_float(value);
 
-    str = (char*)icalmemory_tmp_buffer(40);
+    str = (char*)icalmemory_new_buffer(40);
 
     snprintf(str,40,"%f",data);
 
     return str;
 }
 
-static char* icalvalue_geo_as_ical_string(const icalvalue* value) {
+
+static char* icalvalue_geo_as_ical_string_r(const icalvalue* value) {
 
     struct icalgeotype data;
     char* str;
@@ -1037,35 +1071,38 @@ static char* icalvalue_geo_as_ical_string(const icalvalue* value) {
 
     data = icalvalue_get_geo(value);
 
-    str = (char*)icalmemory_tmp_buffer(80);
+    str = (char*)icalmemory_new_buffer(80);
 
     snprintf(str,80,"%f;%f",data.lat,data.lon);
 
     return str;
 }
 
-static const char* icalvalue_datetimeperiod_as_ical_string(const icalvalue* value) {
+
+static char* icalvalue_datetimeperiod_as_ical_string_r(const icalvalue* value) {
     struct icaldatetimeperiodtype dtp = icalvalue_get_datetimeperiod(value);
 
     icalerror_check_arg_rz( (value!=0),"value");
 
     if(!icaltime_is_null_time(dtp.time)){
-	return icaltime_as_ical_string(dtp.time);
+	return icaltime_as_ical_string_r(dtp.time);
     } else {
-	return icalperiodtype_as_ical_string(dtp.period);
+	return icalperiodtype_as_ical_string_r(dtp.period);
     }
 }
 
-static const char* icalvalue_period_as_ical_string(const icalvalue* value) {
+
+static char* icalvalue_period_as_ical_string_r(const icalvalue* value) {
     struct icalperiodtype data;
     icalerror_check_arg_rz( (value!=0),"value");
     data = icalvalue_get_period(value);
 
-    return icalperiodtype_as_ical_string(data);
+    return icalperiodtype_as_ical_string_r(data);
 
 }
 
-static const char* icalvalue_trigger_as_ical_string(const icalvalue* value) {
+
+static char* icalvalue_trigger_as_ical_string_r(const icalvalue* value) {
 
     struct icaltriggertype data;
 
@@ -1073,15 +1110,25 @@ static const char* icalvalue_trigger_as_ical_string(const icalvalue* value) {
     data = icalvalue_get_trigger(value);
 
     if(!icaltime_is_null_time(data.time)){
-	return icaltime_as_ical_string(data.time);
+	return icaltime_as_ical_string_r(data.time);
     } else {
-	return icaldurationtype_as_ical_string(data.duration);
+	return icaldurationtype_as_ical_string_r(data.duration);
     }   
 
 }
 
 const char*
 icalvalue_as_ical_string(const icalvalue* value)
+{
+	char *buf;
+	buf = icalvalue_as_ical_string_r(value);
+	icalmemory_add_tmp_buffer(buf);
+	return buf;
+}
+
+
+char*
+icalvalue_as_ical_string_r(const icalvalue* value)
 {
     if(value == 0){
 	return 0;
@@ -1090,55 +1137,55 @@ icalvalue_as_ical_string(const icalvalue* value)
     switch (value->kind){
 
     case ICAL_ATTACH_VALUE:
-        return icalvalue_attach_as_ical_string(value);
+        return icalvalue_attach_as_ical_string_r(value);
         
     case ICAL_BINARY_VALUE:
-        return icalvalue_binary_as_ical_string(value);
+        return icalvalue_binary_as_ical_string_r(value);
         
     case ICAL_BOOLEAN_VALUE:
     case ICAL_INTEGER_VALUE:
-        return icalvalue_int_as_ical_string(value);                  
+        return icalvalue_int_as_ical_string_r(value);                  
         
     case ICAL_UTCOFFSET_VALUE:
-        return icalvalue_utcoffset_as_ical_string(value);                  
+        return icalvalue_utcoffset_as_ical_string_r(value);                  
         
     case ICAL_TEXT_VALUE:
-        return icalvalue_text_as_ical_string(value);
+        return icalvalue_text_as_ical_string_r(value);
         
     case ICAL_QUERY_VALUE:
-        return icalvalue_string_as_ical_string(value);
+        return icalvalue_string_as_ical_string_r(value);
         
     case ICAL_STRING_VALUE:
     case ICAL_URI_VALUE:
     case ICAL_CALADDRESS_VALUE:
-        return icalvalue_string_as_ical_string(value);
+        return icalvalue_string_as_ical_string_r(value);
         
     case ICAL_DATE_VALUE:
-        return icalvalue_date_as_ical_string(value);
+        return icalvalue_date_as_ical_string_r(value);
     case ICAL_DATETIME_VALUE:
-        return icalvalue_datetime_as_ical_string(value);
+        return icalvalue_datetime_as_ical_string_r(value);
     case ICAL_DURATION_VALUE:
-        return icalvalue_duration_as_ical_string(value);
+        return icalvalue_duration_as_ical_string_r(value);
         
     case ICAL_PERIOD_VALUE:
-        return icalvalue_period_as_ical_string(value);
+        return icalvalue_period_as_ical_string_r(value);
     case ICAL_DATETIMEPERIOD_VALUE:
-        return icalvalue_datetimeperiod_as_ical_string(value);
+        return icalvalue_datetimeperiod_as_ical_string_r(value);
         
     case ICAL_FLOAT_VALUE:
-        return icalvalue_float_as_ical_string(value);
+        return icalvalue_float_as_ical_string_r(value);
         
     case ICAL_GEO_VALUE:
-        return icalvalue_geo_as_ical_string(value);
+        return icalvalue_geo_as_ical_string_r(value);
         
     case ICAL_RECUR_VALUE:
-        return icalvalue_recur_as_ical_string(value);
+        return icalvalue_recur_as_ical_string_r(value);
         
     case ICAL_TRIGGER_VALUE:
-        return icalvalue_trigger_as_ical_string(value);
+        return icalvalue_trigger_as_ical_string_r(value);
 
     case ICAL_REQUESTSTATUS_VALUE:
-        return icalreqstattype_as_string(value->data.v_requeststatus);
+        return icalreqstattype_as_string_r(value->data.v_requeststatus);
         
     case ICAL_ACTION_VALUE:
     case ICAL_CMD_VALUE:
@@ -1149,14 +1196,14 @@ icalvalue_as_ical_string(const icalvalue* value)
     case ICAL_TRANSP_VALUE:
     case ICAL_CLASS_VALUE:
         if(value->x_value !=0){
-            return icalmemory_tmp_copy(value->x_value);
+            return icalmemory_strdup(value->x_value);
         }
 
-        return icalproperty_enum_to_string(value->data.v_enum);
+        return icalproperty_enum_to_string_r(value->data.v_enum);
         
     case ICAL_X_VALUE: 
 	if (value->x_value != 0)
-            return icalmemory_tmp_copy(value->x_value);
+        return icalmemory_strdup_and_quote(value,value->x_value);
 
     /* FALLTHRU */
 
@@ -1310,9 +1357,12 @@ icalvalue_compare(const icalvalue* a, const icalvalue *b)
 	case ICAL_RECUR_VALUE:
 	{
 	    int r;
-
-	    r =  strcmp(icalvalue_as_ical_string(a),
-			  icalvalue_as_ical_string(b));
+	    char *temp1, *temp2;
+	    temp1 = icalvalue_as_ical_string_r(a);
+	    temp2 = icalvalue_as_ical_string_r(b);
+	    r =  strcmp(temp1, temp2);
+	    free(temp1);
+	    free(temp2);
 
 	    if (r > 0) { 	
 		return ICAL_XLICCOMPARETYPE_GREATER;
@@ -1418,17 +1468,19 @@ int icalvalue_encode_ical_string(const char *szText, char *szEncText, int nMaxBu
     if (value == 0)
         return 0;
     
-    ptr = icalvalue_text_as_ical_string(value);
+    ptr = icalvalue_text_as_ical_string_r(value);
     if (ptr == 0)
         return 0;
     
     if ((int)strlen(ptr) >= nMaxBufferLen)
         {
             icalvalue_free (value);
+	    free(ptr);
             return 0;
         }
 
     strcpy(szEncText, ptr);
+    free(ptr);
 
     icalvalue_free ((icalvalue*)value);
 
